# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/chat_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lv_chat_history = QtWidgets.QListView(self.centralwidget)
        self.lv_chat_history.setMinimumSize(QtCore.QSize(0, 10))
        self.lv_chat_history.setObjectName("lv_chat_history")
        self.verticalLayout.addWidget(self.lv_chat_history)
        self.hl_user_interface = QtWidgets.QHBoxLayout()
        self.hl_user_interface.setContentsMargins(-1, 0, -1, -1)
        self.hl_user_interface.setObjectName("hl_user_interface")
        self.te_chatinput = QtWidgets.QTextEdit(self.centralwidget)
        self.te_chatinput.setMaximumSize(QtCore.QSize(16777215, 80))
        self.te_chatinput.setObjectName("te_chatinput")
        self.hl_user_interface.addWidget(self.te_chatinput)
        self.pb_send = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pb_send.sizePolicy().hasHeightForWidth())
        self.pb_send.setSizePolicy(sizePolicy)
        self.pb_send.setObjectName("pb_send")
        self.hl_user_interface.addWidget(self.pb_send)
        self.verticalLayout.addLayout(self.hl_user_interface)
        MainWindow.setCentralWidget(self.centralwidget)
        self.mb_menubar = QtWidgets.QMenuBar(MainWindow)
        self.mb_menubar.setGeometry(QtCore.QRect(0, 0, 800, 22))
        self.mb_menubar.setObjectName("mb_menubar")
        self.m_menu = QtWidgets.QMenu(self.mb_menubar)
        self.m_menu.setObjectName("m_menu")
        MainWindow.setMenuBar(self.mb_menubar)
        self.sb_statusbar = QtWidgets.QStatusBar(MainWindow)
        self.sb_statusbar.setObjectName("sb_statusbar")
        MainWindow.setStatusBar(self.sb_statusbar)
        self.a_exit = QtWidgets.QAction(MainWindow)
        self.a_exit.setObjectName("a_exit")
        self.actionHide_window = QtWidgets.QAction(MainWindow)
        self.actionHide_window.setObjectName("actionHide_window")
        self.a_new_chat = QtWidgets.QAction(MainWindow)
        self.a_new_chat.setObjectName("a_new_chat")
        self.a_clear_chat = QtWidgets.QAction(MainWindow)
        self.a_clear_chat.setObjectName("a_clear_chat")
        self.a_settings = QtWidgets.QAction(MainWindow)
        self.a_settings.setObjectName("a_settings")
        self.m_menu.addAction(self.a_settings)
        self.m_menu.addAction(self.a_clear_chat)
        self.m_menu.addAction(self.a_exit)
        self.mb_menubar.addAction(self.m_menu.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Sprite AI"))
        self.pb_send.setText(_translate("MainWindow", "Send"))
        self.m_menu.setTitle(_translate("MainWindow", "Menu"))
        self.a_exit.setText(_translate("MainWindow", "Exit"))
        self.a_exit.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionHide_window.setText(_translate("MainWindow", "Hide window"))
        self.a_new_chat.setText(_translate("MainWindow", "New chat"))
        self.a_clear_chat.setText(_translate("MainWindow", "Clear chat"))
        self.a_settings.setText(_translate("MainWindow", "Settings"))
